/*    
 *    sepSAM.cpp		
 *    DiffSplice
 *
 *    Copyright (C) 2012 University of Kentucky and
 *                       Yin Hu
 *
 *    Authors: Yin Hu
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#define UNIX

#ifdef UNIX
#include <fstream>
#include <cstdio>
#include <cstring>
#include <iostream>
#include <cstdlib>
#include <sstream>
#include <vector>
#else
#include <fstream>
#include <stdio.h>
#include <string>
#include <iostream>
#include <stdlib.h> 
#include <sstream>
#include <vector>
#endif

using namespace std;


vector <string> targetChromosome;
//int targetChrNum;

class reads
{
public:
	string readname;
	string field1;
	string field2;
	string field3;
	string field4;
	string field5;
	string XS;

	reads();
};

vector <long> readAlignIndex;
long totalReadNum;
//reads *readAlign[500][1000000];

vector <ofstream*> outputfile;

string lastTimeName;
string dirPrefix;

ofstream abnormal_reads;
ofstream dropped_reads;

reads::reads()
{
	XS = "+";
}



void process(reads* curRead)
{
	//output a single read
	string chromosome, outputfilename;
	unsigned int tmpLoop;

	chromosome = curRead->field2;

	for (tmpLoop = 0; tmpLoop < targetChromosome.size(); ++tmpLoop)
	{
		if (chromosome.compare(targetChromosome[tmpLoop]) == 0)
			break;
	}

	if (tmpLoop >= targetChromosome.size())
	{
		//chromosome not found
		outputfilename = dirPrefix + chromosome + ".txt";

		outputfile.push_back(new ofstream);
		outputfile[tmpLoop]->open(outputfilename.c_str());

		targetChromosome.push_back(chromosome);		
		readAlignIndex.push_back(0);
	}

	readAlignIndex[tmpLoop] += 1;
//	readAlign[tmp][readAlignIndex[tmp]] = curRead;

	*outputfile[tmpLoop] << curRead->readname << "\t" << curRead->field1 << "\t" << curRead->field2 << "\t" << curRead->field3 << "\t" << curRead->field4 << "\t" << curRead->field5 << "\t" << curRead->XS << endl; 


	if (lastTimeName.compare(curRead->readname) != 0)
	{
		lastTimeName = curRead->readname;
		++totalReadNum;
	}

	delete curRead;
	return;
}


void outputRead()
{
	unsigned int tmpLoop;
	string outputfilename;
	ofstream outputChrname, outputDataStat;
	long totalAlignNum = 0;

	outputfilename = dirPrefix + "ChromosomeName.txt";
	outputChrname.open(outputfilename.c_str());
	outputfilename = dirPrefix + "DatasetStat.txt";
	outputDataStat.open(outputfilename.c_str(), fstream::app);

	for (tmpLoop = 0; tmpLoop < targetChromosome.size(); ++tmpLoop)
	{
		outputChrname << targetChromosome[tmpLoop] << '\t' << readAlignIndex[tmpLoop] << endl; 
		totalAlignNum += readAlignIndex[tmpLoop];

		(*outputfile[tmpLoop]).close();
		delete outputfile[tmpLoop];
	}

	outputChrname << "#" << totalReadNum << '\t' << totalAlignNum << endl;

	outputDataStat << totalAlignNum << endl;

	outputChrname.close();
	dropped_reads.close();
	abnormal_reads.close();
	outputDataStat.close();

	return;
}


void parse(string inputfilename)
{
	reads *ptr = NULL;

	ifstream inputfile;
	inputfile.open(inputfilename.c_str());

	string name, curName, notUsedField;
	char optField[10000];
	int tmp;
	stringstream iss;
	string info;
	
	while (inputfile >> name)
	{
		if (name[0] == '@')
		{
			getline(inputfile, info);
			continue;
		}

// 		for (tmp = 0; tmp < 99; tmp++)
// 		{
// 			if (name[tmp] == '/' || name[tmp] == '\0')
// 			{
// 				break;
// 			}
// 		}
// 		name[tmp] = '\0'; 

		ptr = new reads;

		ptr->readname = name;
		inputfile >> ptr->field1;
		inputfile >> ptr->field2;
		inputfile >> ptr->field3;
		inputfile >> ptr->field4;
		inputfile >> ptr->field5;

		for (tmp = 0; tmp < 5; ++tmp)
		{
			inputfile >> notUsedField;
		}
		
		getline(inputfile, info);
		
		if (info.empty() == false)
		{
			iss << info;
			while (iss.getline(optField, 10000, '\t'))
			{
				if (optField[0] == 'X' && optField[1] == 'S')
				{
					ptr->XS = optField[5];
				}
			}
			iss.clear();
		}

		process(ptr);
	}


	inputfile.close();

	return;
}


void initialization()
{
	totalReadNum = 0;
	lastTimeName = "XXXX";

	return;
}


int main(int argc, char* argv[])
{
#ifdef UNIX
	if (argc != 3)
	{
		cout << argv[0] << "\t<filename>\t<target_path>" << endl;
		return 1;
	}

	dirPrefix = argv[2];

	if (dirPrefix.compare("./") == 0)
		dirPrefix = "./data/tmp/";
#else
	dirPrefix = "";
#endif

	string outpurfilename;
	outpurfilename = dirPrefix + "DroppedReads.txt";
	dropped_reads.open(outpurfilename.c_str());
	outpurfilename = dirPrefix + "AbnormalReads.txt";
	abnormal_reads.open(outpurfilename.c_str());

	string inputfilename;
#ifdef UNIX
	inputfilename = argv[1];
#else
	inputfilename = "test.sam";
#endif

	initialization();

	parse(inputfilename);

	outputRead();

	return 0;
}


